#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 Mark Tully <markjtully@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, Gio, ECalendar, EDataServer
from gi.repository import Unity
from time import mktime
import gettext
import datetime
import locale

locale.setlocale(locale.LC_ALL, '')


APP_NAME = 'unity-scope-evolution'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Calendar.Evolution'
UNIQUE_PATH = '/com/canonical/unity/scope/calendar/evolution'

SEARCH_HINT = _('Search Evolution events')
NO_RESULTS_HINT = _('Sorry, there are no Evolution results that match your search.')
PROVIDER_CREDITS = _('')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-evolution.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'result-calendar.svg'
DEFAULT_RESULT_MIMETYPE = 'taglib/rfc822'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

c1 = {'id': 'upcoming',
      'name': _('Upcoming Events'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
c2 = {'id': 'recent',
      'name': _('Recent Events'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1, c2]

FILTERS = []

EXTRA_METADATA = []


def get_events_from_evolution(date=None):
    '''
    Gets a list of events from EDS
    '''
    events = []
    registry = EDataServer.SourceRegistry.new_sync(None)
    sources = EDataServer.SourceRegistry.list_sources(registry, EDataServer.SOURCE_EXTENSION_CALENDAR)
    # Initialize client and open it
    for source in sources:
        client = ECalendar.CalClient.new(source, ECalendar.CalSourceType.EVENT)
        client.open_sync(False, None)
        try:
            ret, values = client.get_object_list_as_comps_sync("#t", None)
            if ret:
                for value in values:
                    dtstart = datetime.datetime.now()
                    # We have to parse the icalstring to get the startdate (for ranking)
                    # We shouldn't have to do this if get_dtstart() worked correctly
                    # At the moment, it returns  a decimal of the memory location of the
                    # start time rather than the start time itself.
                    icalstring = value.get_as_string()
                    lines = icalstring.split('\n')
                    for line in lines:
                        if line.startswith("DTSTART"):
                            date = line[line.index(':') + 1:]
                            try:
                                if 'T' in date:
                                    dtstart = datetime.datetime.strptime(date[:14], '%Y%m%dT%H%M%S')
                                else:
                                    dtstart = datetime.datetime.strptime(date, '%Y%m%d\n')
                            except Exception as error:
                                print('Could not determine start date: %s' % error)

                    sortableevent = []
                    sortableevent.append(dtstart)
                    sortableevent.append(value)
                    sortableevent.append(source.props.display_name)
                    events.append(sortableevent)
        except Exception as e:
            print("Cannot open calendar '%s': %s" % (source.get_display_name(), e))
    return events


def search(search, filters):
    '''
    Search for calendar matching the search string
    '''
    results = []

    events = []
    events = get_events_from_evolution()
    events.sort(key=lambda event: event[0])

    for event in events:
        result = parse_event(search, event)
        if result:
            results.append(result)

    events.sort(key=lambda event: event[0], reverse=True)

    for event in events:
        result = parse_event(search, event)
        if result:
            results.append(result)

    return results


def parse_event(search, event):
    '''
    Parses the passed event and returns a result if the search matches the
    summary or location of the event
    '''
    component = event[1]
    icalstring = event[1].get_as_string()
    lines = icalstring.split('\n')
    dtstart = ""
    dtend = ""
    summary = ""
    location = ""
    uid = ""
    for line in lines:
        if line.startswith("DTSTART"):
            date = line[line.index(':') + 1:]
            try:
                if 'T' in date:
                    dtstart = datetime.datetime.strptime(date[:14], '%Y%m%dT%H%M%S')
                else:
                    dtstart = datetime.datetime.strptime(date, '%Y%m%d\n')
            except Exception as error:
                print('Could not determine start date: %s' % error)
                dtstart = datetime.datetime.now()
        if line.startswith("DTEND"):
            date = line[line.index(':') + 1:]
            try:
                if 'T' in date:
                    dtend = datetime.datetime.strptime(date[:14], '%Y%m%dT%H%M%S')
                else:
                    dtend = datetime.datetime.strptime(date, '%Y%m%d\n')
            except Exception as error:
                print('Could not determine start date: %s' % error)
        if line.startswith("LOCATION"):
            location = line[line.index(':') + 1:]
            location = location.replace('\\', '')
        if line.startswith("UID"):
            uid = line[line.index(':') + 1:]
    summary = ECalendar.CalComponentText()
    component.get_summary(summary)
    summary = summary.value

    now = datetime.datetime.now()
    start = mktime(dtstart.timetuple())
    now = mktime(now.timetuple())
    icon_hint = "/usr/lib/unity-lens-calendar/icons/%s.svg" % dtstart.day

    if start > now:
        group = 0
    else:
        group = 1

    if dtend:
        start = dtstart.strftime('%x %H:%M') + " - " + dtend.strftime('%H:%M')
        if dtstart.strftime('%x %H:%M') == dtend.strftime('%x %H:%M'):
            start = dtstart.strftime('%x %H:%M')
        if dtstart.strftime('%H:%M') == "00:00" and dtend.strftime('%H:%M') == "00:00":
            start = '%s %s' % (dtstart.strftime('%x'), _(" All Day"))
    else:
        start = dtstart.strftime('%x %H:%M')
    if search.lower() in summary.lower():
        return {'uri': uid,
                'icon': icon_hint,
                'category': group,
                'title': summary,
                'comment': "%s\n%s" % (start, location)}

    if search.lower() in location.lower():
        return {'uri': uid,
                'icon': icon_hint,
                'category': group,
                'title': summary,
                'comment': "%s\n%s" % (start, location)}
    return ''

# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                i['metadata'] = {}
                if EXTRA_METADATA:
                    for e in i:
                        for m in EXTRA_METADATA:
                            if m['id'] == e:
                                i['metadata'][e] = i[e]
                i['metadata']['provider_credits'] = GLib.Variant('s', PROVIDER_CREDITS)
                result = Unity.ScopeResult.create(str(i['uri']), str(i['icon']),
                                                  i['category'], i['result_type'],
                                                  str(i['mimetype']), str(i['title']),
                                                  str(i['comment']), str(i['dnd_uri']),
                                                  i['metadata'])
                result_set.add_result(result)
        except Exception as error:
            print(error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
        #if FILTERS:
        #
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se


def load_scope():
    return Scope()
